#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QStringList>
#include <QFileDialog>
#include <math.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    int devices = setupESCAPI();
    char devNameBuf[256];
    int i;
    QString devName;
    for(i = 0; i < devices; i++)
    {
        getCaptureDeviceName(i, devNameBuf, 256);
        devName = devNameBuf;
        ui->cbVideoSource->addItem(devName, QVariant(i));
    }

    escapiCapture.mWidth = 320;
    escapiCapture.mHeight = 240;
    escapiCapture.mTargetBuf = new int[escapiCapture.mWidth * escapiCapture.mHeight];

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pbCapture_clicked()
{
    int device;
    device = ui->cbVideoSource->currentIndex();
    capturedImage = getCaptureImage(device);
    ui->lblCapture->setPixmap(QPixmap::fromImage(capturedImage));
}

QImage MainWindow::getCaptureImage(int device)
{
    initCapture(0, &escapiCapture);
    doCapture(device);

    //TODO needs timer check for failure modes
    while (isCaptureDone(device) == 0){}

    //copy escapi capture buffer to a QImage
    QImage img( escapiCapture.mWidth, escapiCapture.mHeight, QImage::Format_ARGB32);
    int* imgData = (int*)(img.bits());
    int x,y;
    for (x = 0; x < escapiCapture.mWidth; x++)
    {
        for (y = 0; y < escapiCapture.mHeight; y++)
        {
            imgData[x + escapiCapture.mWidth*y] = escapiCapture.mTargetBuf[x + escapiCapture.mWidth*y];
        }
    }

    return (img);
}


void MainWindow::on_pbRedOnly_clicked()
{
    int* imgData = (int*)(capturedImage.bits());
    int x,y, pixel;
    for (x = 0; x < capturedImage.width(); x++)
    {
        for (y = 0; y < capturedImage.height(); y++)
        {
            pixel = x + capturedImage.width()*y;
            imgData[pixel] = imgData[pixel] & 0xffff0000;
        }
    }
    ui->lblCapture->setPixmap(QPixmap::fromImage(capturedImage));
}

void MainWindow::on_pbBestRedOnly_clicked()
{

    capturedImage = getBestRed(capturedImage);
    ui->lblCapture->setPixmap(QPixmap::fromImage(capturedImage));
}

QImage MainWindow::getBestRed(QImage rawImg)
{
    QImage img = rawImg;
    unsigned int* imgData = (unsigned int*)(img.bits());
    unsigned int x,y, pixel;
    unsigned int maxRed, maxPixel;
    for (x = 0; x < img.width(); x++)
    {
        maxRed = 0;
        for (y = 0; y < img.height(); y++)
        {
            pixel = x + img.width()*y;
            if ((imgData[pixel] & 0xffff0000) > maxRed)
            {
                maxRed = imgData[pixel] & 0xffff0000;
                maxPixel = pixel;
            }
        }

        for (y = 0; y < img.height(); y++)
        {
            pixel = x + img.width()*y;
            if (pixel != maxPixel) imgData[pixel] = 0xff000000;
        }
    }

    return(img);
}


double MainWindow::GetDistance(double y, double frameHeight, double camFov, double camAngle, double camHeight)
{
    double angle = ((frameHeight - y) / (frameHeight)) * camFov + (camAngle - camFov/2.0);
    angle = 2*3.14159265358*(angle/360.0);
    return(camHeight*tan(angle));
}

void MainWindow::populateGrid(double min, double max, QImage& distMap)
{
    int scale = distMap.height() / (max - min);

    int grid_idx;
    int y, x;
    unsigned int* imgData = (unsigned int*)(distMap.bits());
    for(grid_idx = 0, y =0; y < distMap.height(); grid_idx++, y = grid_idx*scale)
    {
        for(x = 0; x < distMap.width(); x++)
        {
            imgData[x + y*distMap.width()] = 0xff808080;
        }
    }
}

void MainWindow::on_pbAnalyzeCapture_clicked()
{
    QImage bestRed = getBestRed(capturedImage);
    QImage distMap(ui->lblDistMap->width(), ui->lblDistMap->height(), QImage::Format_ARGB32);

    int mapHeight = ui->lblDistMap->height();
    int mapWidth = ui->lblDistMap->width();
    distMap.fill(0xff000000);
    double mapMin = 16.0, mapMax = 32.0;
    populateGrid(mapMin, mapMax, distMap);

    unsigned int* imgData = (unsigned int*)(bestRed.bits());
    unsigned int* distData = (unsigned int*)(distMap.bits());
    unsigned int x,y, pixel;
    double dist;
    int distY;

    for (x = 0; x < bestRed.width(); x++)
    {
        for (y = 0; y < bestRed.height(); y++)
        {
            pixel = x + bestRed.width()*y;
            if ((imgData[pixel] & 0x00ff0000) != 0)
            {
                if (y == 0) continue;
                dist = GetDistance(y,
                                   bestRed.height(),
                                   ui->CameraFOV->value(),
                                   ui->CameraAngle->value(),
                                   ui->CameraHeight->value());
                distY = mapHeight - ((dist - mapMin) / (mapMax - mapMin)) * mapHeight;
                if (distY < mapHeight && distY >= 0)
                {
                    distData[x + distY*mapWidth] = 0xffffffff;
                }
                break;
            }
        }
    }

    ui->lblDistMap->setPixmap(QPixmap::fromImage(distMap));
}

void MainWindow::on_pbSave_clicked()
{
    QStringList fileNames;
    QFileDialog sv(this, tr("Save Image"));
    sv.setAcceptMode(QFileDialog::AcceptSave);
    if (sv.exec())
        fileNames = sv.selectedFiles();

    capturedImage.save(fileNames.first());
}

void MainWindow::on_pbLoad_clicked()
{
    QStringList fileNames;
    QFileDialog ld(this, tr("Load Image"));
    if (ld.exec())
        fileNames = ld.selectedFiles();

    capturedImage.load(fileNames.first());
    ui->lblCapture->setPixmap(QPixmap::fromImage(capturedImage));
}


